use leptos::prelude::*;

pub struct HeaderItems {
    pub left: Vec<HeaderItem>,
    pub middle: Vec<HeaderItem>,
    pub right: Vec<HeaderItem>,
}

pub struct HeaderItem {
    pub text: String,
    pub link: Option<String>,
}

/// Header with background color etc.
#[component]
pub fn StyledHeader(items: HeaderItems) -> impl IntoView {
    let fancy = false;

    if fancy {
        view! {
            <div class="flex">
                // Left gradient chunk
                <div class="flex-grow">
                    <div class="h-2/5" style="background: #eaac53" />
                    <div
                        class="h-3/5"
                        style="background: linear-gradient(to bottom left, #eaac53 49.5%, rgb(15 23 42) 50.5%)"
                    />
                </div>

                <div class="flex-none container mx-auto text-black" style="background: #eaac53">
                    <Header items />
                </div>

                // Right gradient chunk
                <div class="flex-grow" style="background: #eaac53" />
            </div>
            <div class="flex">
                // Left gradient chunk
                <div class="flex-grow" />

                <div class="flex-none container mx-auto">
                    // Background color gradient
                    <div
                        class="h-6"
                        style="background: linear-gradient(to bottom left, #eaac53 49.5%, rgb(15 23 42) 50.5%)"
                    />
                </div>

                // Right gradient chunk
                <div class="flex-grow">
                    <div class="h-4/5" style="background: #eaac53" />
                    <div
                        class="h-1/5"
                        style="background: linear-gradient(to bottom right, #eaac53 49.5%, rgb(15 23 42) 50.5%)"
                    />
                </div>
            </div>
        }
        .into_any()
    } else {
        view! {
            <div class="bg-orange-300 text-black border-b-2 border-b-orange-400">
                <div class="container mx-auto" >
                    <Header items />
                </div>
            </div>
        }
        .into_any()
    }
}

/// Function header without styling
#[component]
pub fn Header(items: HeaderItems) -> impl IntoView {
    let HeaderItems { left, middle, right } = items;

    view! {
        <div class="grid grid-cols-[1fr_3fr_1fr] text-xl font-semibold p-4">
            // Left side of header
            <div class="justify-self-start">
                <Items items=left />
            </div>

            // Expanding space in the middle
            <div class="justify-self-center font-semibold">
                <Items items=middle />
            </div>

            // Right side of header
            <div class="justify-self-end">
                <Items items=right />
            </div>
        </div>
    }
}

#[component]
fn Items(items: Vec<HeaderItem>) -> impl IntoView {
    let items = items.into_iter().map(|item| view! { <Item item /> }).collect_view();
    view! { <div class="flex gap-4">{items}</div> }
}

#[component]
fn Item(item: HeaderItem) -> impl IntoView {
    let text = item.text;
    let link = item.link;

    if let Some(link) = link {
        view! {
            <a href=link>
                <span class="whitespace-nowrap">{text}</span>
            </a>
        }
        .into_any()
    } else {
        view! { <span class="whitespace-nowrap">{text}</span> }.into_any()
    }
}
